package com.miya.draw.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.miya.draw.entity.AccountPrize;
import com.miya.draw.enums.PrizeTypeEnum;
import com.miya.draw.mapper.AccountPrizeMapper;
import com.miya.draw.service.IAccountPrizeService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author jack
 * @since 2021-05-05
 */
@Service
public class AccountPrizeServiceImpl extends ServiceImpl<AccountPrizeMapper, AccountPrize> implements IAccountPrizeService {

    @Resource
    private AccountPrizeMapper accountPrizeMapper;

    @Override
    public Integer gainAlreadyDrawCount(String phone, LocalDateTime start, LocalDateTime end) {
        LambdaQueryWrapper<AccountPrize> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(AccountPrize::getAccountPhone, phone)
                .between(AccountPrize::getCreateTime, start, end);
        return accountPrizeMapper.selectCount(wrapper);
    }

    @Override
    public List<AccountPrize> listMinePrize(String phone) {
        LambdaQueryWrapper<AccountPrize> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(AccountPrize::getAccountPhone, phone);
        wrapper.ne(AccountPrize::getPrizeType, PrizeTypeEnum.NO_COUPON.getCode());
        wrapper.orderByDesc(AccountPrize::getId);
        return accountPrizeMapper.selectList(wrapper);
    }

    @Override
    public List<AccountPrize> listAllWinningInfo(Integer size) {
        LambdaQueryWrapper<AccountPrize> wrapper = new LambdaQueryWrapper<>();
        wrapper.ne(AccountPrize::getPrizeType, PrizeTypeEnum.NO_COUPON.getCode());
        wrapper.orderByAsc(AccountPrize::getPrizeType);
        wrapper.last("limit " + size);
        return accountPrizeMapper.selectList(wrapper);
    }

    @Override
    public Integer countJoinActivityPeople() {
        LambdaQueryWrapper<AccountPrize> wrapper = new LambdaQueryWrapper<>();
        return accountPrizeMapper.selectCount(wrapper);
    }

}
