package com.miya.draw.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.miya.draw.config.constant.ConStants;
import com.miya.draw.entity.ActivityGoods;
import com.miya.draw.enums.YesOrNoEnum;
import com.miya.draw.mapper.ActivityGoodsMapper;
import com.miya.draw.service.IActivityGoodsService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author jack
 * @since 2021-05-05
 */
@Service
@Slf4j
public class ActivityGoodsServiceImpl extends ServiceImpl<ActivityGoodsMapper, ActivityGoods> implements IActivityGoodsService {

    @Resource
    private ActivityGoodsMapper activityGoodsMapper;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Override
    public List<String> listAllActivityGoods() {
        List<String> goodList = new ArrayList<>();
        try {
            Object object = redisTemplate.opsForValue().get(buildKey());
            if (Objects.nonNull(object)) {
                return (List<String>) object;
            }
        } catch (Exception e) {
            log.error("从缓存读取活动商品错误: {} ，继续执行数据库操作", e.getMessage());
        }
        LambdaQueryWrapper<ActivityGoods> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(ActivityGoods::getValidState, YesOrNoEnum.YES.getCode());
        List<ActivityGoods> activityGoodsList = activityGoodsMapper.selectList(wrapper);
        if (CollectionUtils.isNotEmpty(activityGoodsList)) {
            goodList = activityGoodsList.stream().map(ActivityGoods::getCode).collect(Collectors.toList());
            try {
                redisTemplate.opsForValue().setIfAbsent(buildKey(), goodList, Duration.ofHours(1));
            } catch (Exception e) {
                log.error("往缓存储存活动商品错误: {} ，继续执行操作", e.getMessage());
            }
        }

        return goodList;
    }

    /**
     * 获得获得商品的key
     *
     * @return
     */
    private String buildKey() {
        return ConStants.SHIZU_DRAW_PREFIX + ConStants.SHIZU_ACTIVITY_GOOD;
    }
}
