package com.miya.draw.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.miya.draw.config.constant.ConStants;
import com.miya.draw.entity.ActivityStore;
import com.miya.draw.enums.YesOrNoEnum;
import com.miya.draw.mapper.ActivityStoreMapper;
import com.miya.draw.service.IActivityStoreService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;


/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author jack
 * @since 2021-05-05
 */
@Slf4j
@Service
public class ActivityStoreServiceImpl extends ServiceImpl<ActivityStoreMapper, ActivityStore> implements IActivityStoreService {

    @Resource
    private ActivityStoreMapper activityStoreMapper;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Override
    public List<String> listAllActivityStoreId() {
        List<String> storeIds = new ArrayList<>();
        try {
            Object object = redisTemplate.opsForValue().get(buildKey());
            if (Objects.nonNull(object)) {
                return (List<String>) object;
            }
        } catch (Exception e) {
            log.error("从缓存读取活动门店错误: {} ，继续执行数据库操作", e.getMessage());
        }
        LambdaQueryWrapper<ActivityStore> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(ActivityStore::getValidState, YesOrNoEnum.YES.getCode());
        List<ActivityStore> storeList = activityStoreMapper.selectList(wrapper);
        if (CollectionUtils.isNotEmpty(storeList)) {
            storeIds = storeList.stream().map(ActivityStore::getStoreId).collect(Collectors.toList());
            try {
                redisTemplate.opsForValue().setIfAbsent(buildKey(), storeIds, Duration.ofHours(1));
            } catch (Exception e) {
                log.error("往缓存储存活动门店错误: {} ，继续执行操作", e.getMessage());
            }
        }

        return storeIds;
    }

    /**
     * 获得获得门店的key
     *
     * @return
     */
    private String buildKey() {
        return ConStants.SHIZU_DRAW_PREFIX + ConStants.SHIZU_ACTIVITY_STOREID;
    }
}
